# -*- coding: utf-8 -*-
""" TcEx Metrics Module """
import json

class TcExMetricsV2(object):
    """TcEx Metrics Class."""

    def __init__(self, tcex, name, description, data_type, interval, keyed=False):
        """ Initialize class properties

        Args:
            name: The name for the metric.
            description: The description of the metric.
            data_type: The type of metric: Sum, Count, Min, Max, First, Last, and Average
            interval: The metric interval: Hourly, Daily, Weekly, Monthly, and Yearly
            keyed: Indicates whether the data will have a keyed value.
        """
        self._tcex = tcex
        self._metric_data_type = data_type
        self._metric_description = description
        self._metric_id = None
        self._metric_interval = interval
        self._metric_keyed = keyed
        self._metric_name = name

        if not self.metric_find():
            self.metric_create()

    # @property
    # def metric_exists(self):
    #     """ Validate that the metric Id provided is still valid
    #     """
    #     if self._metric_id is not None:
    #         resource = self.metric_resource
    #         resource.metric_id(self._metric_id)
    #         results = resource.request()
    #         if results.get('response').status_code == 200:
    #             return True
    #     return False

    @property
    def metric_resource(self):
        """ Return metric resource """
        return self._tcex.resource('CustomMetric')

    def metric_create(self):
        """Create the defined metric
        """
        resource = self.metric_resource
        resource.http_method = 'POST'
        body = {
            'dataType' : self._metric_data_type,
            'description' : self._metric_description,
            'interval' : self._metric_interval,
            'name' : self._metric_name,
            'keyedValues' : self._metric_keyed
        }
        self._tcex.log.debug('metric body: {}'.format(json.dumps(body)))
        resource.body = json.dumps(body)
        results = resource.request()

        if results.get('status') == 'Success':
            self._metric_id = results.get('data', {}).get('id')
            self._tcex.log.debug('metric data: {}'.format(results.get('data')))
            self._tcex.log.debug('metric id: {}'.format(self._metric_id))
        else:
            err = u'Failed to add metric ({})'.format(results.get('response').text)
            self._tcex.log.error(err)
            raise RuntimeError(err)

    def metric_find(self):
        """Report Metrics"""
        resource = self.metric_resource
        for results in resource:
            if results.get('status') != 'Success':
                err = 'Error during while retrieving metrics.'
                self._tcex.log.error(err)
                continue

            for metric in results.get('data'):
                if metric.get('name') == self._metric_name:
                    self._metric_id = metric.get('id')
                    info = 'found metric with name "{}" and Id {}.'
                    self._tcex.log.info(info.format(self._metric_name, self._metric_id))
                    return True

        return False

    def add(self, value, date=None, return_value=False, key=None):
        """Add Metrics

        Args:
            value: The value of the metric.
            date: The optional date of the metric.
            return_value: Tell the API to return the updates metric value.
            key: The key value for keyed metrics.

        Return:
            (None|Dict): A None value if return value was not requested or a dict.
        """
        response = None
        if self._metric_id is None:
            err = 'No metric ID found for "{}".'.format(self._metric_name)
            self._tcex.log.error(err)
            raise RuntimeError(err)
        resource = self.metric_resource
        resource.http_method = 'POST'
        if return_value:
            resource.add_payload('returnValue', 'true')
        resource.data(self._metric_id)
        body = {
            'value': value
        }
        if date is not None:
            body['date'] = date
        if key is not None:
            body['name'] = key
        self._tcex.log.debug('metric data: {}'.format(json.dumps(body)))
        resource.body = json.dumps(body)
        results = resource.request()

        if results.get('response').status_code == 200:
            response = results.get('response').json()
        elif results.get('response').status_code == 204:
            pass
        else:
            err = u'Failed to add metric ({})'.format(results.get('response').text)
            self._tcex.log.error(err)
            self._tcex.message_tc(err)
            raise RuntimeError(err)

        return response

    def add_keyed(self, value, key, date=None, return_value=False):
        """Add keyed metrics

        Args:
            value: The value of the metric.
            key: The key value for keyed metrics.
            date: The optional date of the metric.
            return_value: Tell the API to return the updates metric value.

        Return:
            (None|Dict): A None value if return value was not requested or a dict.
        """
        return self.add(value, date, return_value, key)



# -*- coding: utf-8 -*-
""" TcEx Framework KeyValue Module """
from builtins import str
try:
    from urllib import quote  # Python 2
except ImportError:
    from urllib.parse import quote  # Python 3


class TcExKeyValue(object):
    """Update Redis via ThreatConnect API Wrapper"""

    def __init__(self, tcex, rhash):
        """ """
        self._tcex = tcex
        self._hash = rhash
        self._r = tcex.request_tc()

    def create(self, key, value):
        """Create key/value pair in Redis"""
        key = quote(key, safe='~')
        self._r.body = value
        self._r.content_type = 'application/octet-stream'
        self._r.http_method = 'PUT'
        self._r.url = '{}/internal/playbooks/keyValue/{}'.format(
            self._tcex.default_args.tc_api_path, key)
        response = self._r.send()
        return response.content

    def delete(self, key):
        """Delete is not supported in API Wrapper"""
        return None

    def read(self, key):
        """Read data from Redis for the provided key"""
        key = quote(key, safe='~')
        self._r.url = '{}/internal/playbooks/keyValue/{}'.format(
            self._tcex.default_args.tc_api_path, key)
        response = self._r.send()

        data = response.content
        if data is not None and not isinstance(data, str):
            data = str(response.content, 'utf-8')
        return data



# -*- coding: utf-8 -*-
""" TcEx Utilities Class """
from datetime import datetime
import calendar
import re

from pytz import timezone
from dateutil import parser
from dateutil.relativedelta import relativedelta
from tzlocal import get_localzone
import parsedatetime as pdt


class TcExUtils():
    """TcEx framework utilities."""

    def __init__(self, tcex):
        """ Initialize class data

        Initialize parent class for default values and logging method.
        """
        self._tcex = tcex

    def any_to_datetime(self, time_input, tz=None):
        """ Return datetime object from multiple formats

            Formats:

            #. Human Input (e.g 30 days ago, last friday)
            #. ISO 8601 (e.g. 2017-11-08T16:52:42Z)
            #. Loose Date format (e.g. 2017 12 25)
            #. Unix Time/Posix Time/Epoch Time (e.g. 1510686617 or 1510686617.298753)

        Args:
            time_input (string): The time input string (see formats above).
            tz (string): The time zone for the returned data.

        Returns:
            (datetime.datetime): Python datetime.datetime object.
        """

        # handle timestamp (e.g. 1510686617 or 1510686617.298753)
        dt_value = self.unix_time_to_datetime(time_input, tz)

        # handle ISO or other formatted date (e.g. 2017-11-08T16:52:42Z,
        # 2017-11-08T16:52:42.400306+00:00)
        if dt_value is None:
            dt_value = self.date_to_datetime(time_input, tz)

        # handle human readable relative time (e.g. 30 days ago, last friday)
        if dt_value is None:
            dt_value = self.human_date_to_datetime(time_input, tz)

        # if all attempt to convert fail raise an error
        if dt_value is None:
            raise RuntimeError('Could not format input ({}) to datetime string.'.format(time_input))

        return dt_value

    def date_to_datetime(self, time_input, tz=None):
        """ Convert ISO 8601 and other date strings to datetime.datetime type.

        Args:
            time_input (string): The time input string (see formats above).
            tz (string): The time zone for the returned data.

        Returns:
            (datetime.datetime): Python datetime.datetime object.
        """
        dt = None
        try:
            dt = parser.parse(time_input)
            # don't covert timezone if dt timezone already in the correct timezone
            if tz is not None and tz != dt.tzname():
                if dt.tzinfo is None:
                    self._tcex.log.info(
                        'Assuming local time for naive datetime {}.'.format(str(dt)))
                    dt = dt.replace(tzinfo=timezone(get_localzone().zone))  # required for py2.x
                dt = dt.astimezone(timezone(tz))
        except ValueError:
            pass

        return dt

    def format_datetime(self, time_input, tz=None, date_format=None):
        """ Return timestamp from multiple input formats.

            Formats:

            #. Human Input (e.g 30 days ago, last friday)
            #. ISO 8601 (e.g. 2017-11-08T16:52:42Z)
            #. Loose Date format (e.g. 2017 12 25)
            #. Unix Time/Posix Time/Epoch Time (e.g. 1510686617 or 1510686617.298753)

        .. note:: To get a unix timestamp format use the strftime format **%s**. While Python
                  does not properly support this the method has functionality to handle this
                  use case.

        Args:
            time_input (string): The time input string (see formats above).
            tz (string): The time zone for the returned data.
            date_format (string): The strftime format to use, ISO by default.

        Returns:
            (string): Formatted datetime string.
        """

        # handle timestamp (e.g. 1510686617 or 1510686617.298753)
        dt_value = self.any_to_datetime(time_input, tz)

        # format date
        if date_format == '%s':
            dt_value = calendar.timegm(dt_value.timetuple())
        elif date_format:
            dt_value = dt_value.strftime(date_format)
        else:
            dt_value = dt_value.isoformat()

        return dt_value

    def human_date_to_datetime(self, time_input, tz=None, source_datetime=None):
        """ Convert human readable date (e.g. 30 days ago) to datetime.datetime using
            parsedatetime module.

        Examples:

        * August 25th, 2008
        * 25 Aug 2008
        * Aug 25 5pm
        * 5pm August 25
        * next saturday
        * tomorrow
        * next thursday at 4pm
        * at 4pm
        * eod
        * tomorrow eod
        * eod tuesday
        * eoy
        * eom
        * in 5 minutes
        * 5 minutes from now
        * 5 hours before now
        * 2 hours before noon
        * 2 days from tomorrow

        Args:
            time_input (string): The time input string (see formats above).
            tz (string): The time zone for the returned datetime.
            source_datetime (datetime.datetime): The reference or source datetime.

        Returns:
            (datetime.datetime): Python datetime.datetime object.
        """

        cal = pdt.Calendar()
        tzinfo = None
        src_tzname = None
        if source_datetime is not None:
            tzinfo = source_datetime.tzinfo
            src_tzname = source_datetime.tzname()
        dt, status = cal.parseDT(time_input, sourceTime=source_datetime, tzinfo=tzinfo)
        if tz is not None:  # don't add tz if no tz value is passed
            if dt.tzinfo is None:
                self._tcex.log.info('Assuming local time for naive datetime {}.'.format(str(dt)))
                dt = dt.replace(tzinfo=timezone(get_localzone().zone))  # required for py2.x
            # don't covert timezone if source timezone already in the correct timezone
            if tz != src_tzname:
                dt = dt.astimezone(timezone(tz))
        if status == 0:
            dt = None

        return dt

    def timedelta(self, time_input1, time_input2):
        """ Calculates time delta between two time expressions.

        Args:
            time_input1 (string): The time input string (see formats above).
            time_input2 (string): The time input string (see formats above).

        Returns:
            (dict): Dict with delta values.
        """
        time_input1 = self.any_to_datetime(time_input1)
        time_input2 = self.any_to_datetime(time_input2)

        diff = (time_input1 - time_input2)  # timedelta
        delta = relativedelta(time_input1, time_input2)  # relativedelta

        # totals
        total_months = (delta.years * 12) + delta.months
        total_weeks = (delta.years * 52) + delta.weeks
        total_days = diff.days  # handles leap days
        total_hours = (total_days * 24) + delta.hours
        total_minutes = (total_hours * 60) + delta.minutes
        total_seconds = (total_minutes * 60) + delta.seconds
        total_microseconds = (total_seconds * 1000) + delta.microseconds
        return {
            'datetime_1': time_input1.isoformat(),
            'datetime_2': time_input2.isoformat(),
            'years': delta.years,
            'months': delta.months,
            'weeks': delta.weeks,
            'days': delta.days,
            'hours': delta.hours,
            'minutes': delta.minutes,
            'seconds': delta.seconds,
            'microseconds': delta.microseconds,
            'total_months': total_months,
            'total_weeks': total_weeks,
            'total_days': total_days,
            'total_hours': total_hours,
            'total_minutes': total_minutes,
            'total_seconds': total_seconds,
            'total_microseconds': total_microseconds
        }

    @staticmethod
    def to_bool(value):
        """ Convert string value to bool """
        bool_value = False
        if str(value).lower() in ['1', 'true']:
            bool_value = True
        return bool_value

    @staticmethod
    def unix_time_to_datetime(time_input, tz=None):
        """ Convert (unix time|epoch time|posix time) in format of 1510686617 or 1510686617.298753
            to datetime.datetime type.

        .. note:: This method assumes UTC for all inputs.

        .. note:: This method only accepts a 9-10 character time_input.

        Args:
            time_input (string): The time input string (see formats above).
            tz (string): The time zone for the returned datetime (e.g. UTC).

        Returns:
            (datetime.datetime): Python datetime.datetime object.
        """
        dt = None
        if re.compile(r'^[0-9]{9,10}(?:\.[0-9]{0,10})?$').findall(str(time_input)):
            dt = datetime.fromtimestamp(float(time_input), tz=timezone('UTC'))
            # don't covert timezone if dt timezone already in the correct timezone
            if tz is not None and tz != dt.tzname():
                dt = dt.astimezone(timezone(tz))

        return dt


"""
>>> from pytz import timezone
>>> from datetime import datetime
>>> time_input = 1229084481
>>> dt = datetime.fromtimestamp(float(time_input), tz=timezone('UTC'))
>>> dt.isoformat()
'2008-12-12T12:21:21+00:00'
>>> tz.normalize(dt).isoformat()
'2008-12-12T06:21:21-06:00'
>>> dt.astimezone(timezone('US/Central'))
datetime.datetime(2008, 12, 12, 6, 21, 21, tzinfo=<DstTzInfo 'US/Central' CST-1 day, 18:00:00 STD>)
>>> dt.astimezone(timezone('US/Central')).isoformat()
'2008-12-12T06:21:21-06:00'
"""



# -*- coding: utf-8 -*-
""" TcEx Framework Redis Module """
from builtins import str

import redis


class TcExRedis(object):
    """Create/Read Data in/from Redis"""

    def __init__(self, host, port, rhash):
        """ """
        self._hash = rhash
        # self._r = redis.StrictRedis(host=host, port=port, db=0)
        self._r = redis.StrictRedis(host=host, port=port)

    def create(self, key, value):
        """Create key/value pair in Redis"""
        return self._r.hset(self._hash, key, value)

    def delete(self, key):
        """Delete data from Redis for the provided key"""
        return self._r.hdel(self._hash, key)

    def read(self, key):
        """Read data from Redis for the provided key"""
        data = self._r.hget(self._hash, key)
        if data is not None and not isinstance(data, str):
            data = str(self._r.hget(self._hash, key), 'utf-8')
        return data
